import 'dart:async';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:flutter/foundation.dart';
import 'dart:io';
import 'package:wowit/interfaces/mixins/handle_exception_mixin.dart';
import 'package:wowit/interfaces/mixins/online_status_mixin.dart';

class WowConnectivity with HandleExceptionMixin {
  WowConnectivity._() {
    Connectivity().onConnectivityChanged.listen(_checkConnection);
    _loopCheckOnline();
  }

  factory WowConnectivity() => _instance;

  static final WowConnectivity _instance = WowConnectivity._();

  /// | Address        | Provider   | Info                                            |
  /// |:---------------|:-----------|:------------------------------------------------|
  /// | 1.1.1.1        | CloudFlare | https://1.1.1.1                                 |
  /// | 1.0.0.1        | CloudFlare | https://1.1.1.1                                 |
  /// | 8.8.8.8        | Google     | https://developers.google.com/speed/public-dns/ |
  /// | 8.8.4.4        | Google     | https://developers.google.com/speed/public-dns/ |
  /// | 208.67.222.222 | OpenDNS    | https://use.opendns.com/                        |
  /// | 208.67.220.220 | OpenDNS    | https://use.opendns.com/                        |
  static final List<InternetAddress> addresses = List.unmodifiable([
    InternetAddress('1.1.1.1'),
    //InternetAddress('1.0.0.1'),
    InternetAddress('8.8.8.8'),
    //InternetAddress('8.8.4.4'),
    InternetAddress('208.67.222.222'),
    //InternetAddress('208.67.220.220'),
  ]);

  static const int port = 53;

  static const Duration timeout = Duration(seconds: 10);

  static const Duration timeloop = Duration(seconds: 10);

  static final Set<OnlineStatusMixin> _list = {};

  static Timer? _timerHandle;

  bool _isOnline = true;
  bool get isOnline => _isOnline;

  void initialise(OnlineStatusMixin controller) async {
    _list.add(controller);
    controller.isOnline = _instance._isOnline;
  }

  static void _checkConnection(
    ConnectivityResult connectivityResult, [
    bool debug = true,
  ]) async {
    bool isOnline = true;
    try {
      bool checkHasInternet = await _instance.hasConnectionInternet(debug);
      isOnline = checkHasInternet;
    } catch (e) {
      _instance.handelException(e, subTitle: '_checkConnection');
      isOnline = false;
    }
    if (debug) {
      if (kDebugMode) {
        print(
          '_checkConnection: isOnline = $isOnline',
        );
      }
    }
    if (isOnline != _instance._isOnline) {
      _instance._isOnline = isOnline;
      for (var controller in _list) {
        if (kDebugMode) {
          print(
            'Connectivity reload connect ${controller.runtimeType}: isOnline = $isOnline',
          );
        }
        controller.isOnline = isOnline;
      }
    }
  }

  static void _loopCheckOnline() async {
    _timerHandle?.cancel();
    ConnectivityResult connectivityResult =
        await Connectivity().checkConnectivity();

    _checkConnection(connectivityResult, false);

    if (_list.isEmpty) return;
    _timerHandle = Timer(timeloop, _loopCheckOnline);
  }

  void dispose(OnlineStatusMixin controller) {
    _list.remove(controller);
  }

  Future<bool> hasConnectionInternet([bool debug = true]) async {
    List<Future<bool>> requests = [];
    for (var address in addresses) {
      requests.add(isHostReachable(address, debug));
    }
    var listCheck = await Future.wait(requests);

    return listCheck.contains(true);
  }

  Future<bool> isHostReachable(
    InternetAddress host, [
    bool debug = true,
  ]) async {
    Socket? sock;
    try {
      sock = await Socket.connect(
        host,
        port,
        timeout: timeout,
      );
      sock.destroy();
      if (debug) {
        if (kDebugMode) {
          print('connect Internet Address: $host = true');
        }
      }
      return true;
    } catch (e) {
      sock?.destroy();
      if (kDebugMode) {
        print('connect Internet Address: $host = false');
      }
      return false;
    }
  }
}
